#include "MainWindow.h"
#include "Gui.h"

extern Gui *gui;

MainWindow::MainWindow(GtkWidget *wnd) 
	: Window(wnd) {
	this->fileName = NULL;
	g_signal_connect(G_OBJECT(this->wnd), "delete-event", G_CALLBACK(MainWindow::onDeleteEvent), NULL);
	g_signal_connect(G_OBJECT(this->wnd), "destroy", G_CALLBACK(MainWindow::onDestroy), NULL);
	
	this->webView = WEBKIT_WEB_VIEW(webkit_web_view_new());
	GtkWidget* scrolledWindow = gui->getWidget("scrolledWindow");
	gtk_container_add(GTK_CONTAINER(scrolledWindow), GTK_WIDGET(this->webView));
	gtk_widget_show(GTK_WIDGET(this->webView));
	g_signal_connect(this->webView, "close-web-view", G_CALLBACK(MainWindow::closeWebView), this->wnd);
	g_signal_connect(this->webView, "notify::load-status", G_CALLBACK(MainWindow::webViewLoadStatus), this);
	
	this->homePage = "http://www.webkitgtk.org/";
	
	this->edtUrl = gui->getWidget("edtURL");
	gtk_entry_set_text(GTK_ENTRY(this->edtUrl), this->homePage);
	g_signal_connect(this->edtUrl, "activate", G_CALLBACK(MainWindow::edtUrlActivated), this);
	
	GtkWidget* btnBack = gui->getWidget("btnBack");
	g_signal_connect(btnBack, "clicked", G_CALLBACK(MainWindow::btnBackClicked), this);
	GtkWidget* btnForward = gui->getWidget("btnForward");
	g_signal_connect(btnForward, "clicked", G_CALLBACK(MainWindow::btnForwardClicked), this);
	GtkWidget* btnGo = gui->getWidget("btnGo");
	g_signal_connect(btnGo, "clicked", G_CALLBACK(MainWindow::btnGoClicked), this);
	GtkWidget* tbOpen = gui->getWidget("tbOpen");
	g_signal_connect(tbOpen, "clicked", G_CALLBACK(MainWindow::tbOpenClicked), this);
	GtkWidget* tbHome = gui->getWidget("tbHome");
	g_signal_connect(tbHome, "clicked", G_CALLBACK(MainWindow::tbHomeClicked), this);
	GtkWidget* tbClose = gui->getWidget("tbClose");
	g_signal_connect(tbClose, "clicked", G_CALLBACK(MainWindow::tbCloseClicked), this);
	GtkWidget* tbAbout = gui->getWidget("tbAbout");
	g_signal_connect(tbAbout, "clicked", G_CALLBACK(MainWindow::tbAboutClicked), this);

	GtkWidget* menuItem = gui->getWidget("mnuNowy");
	g_signal_connect(menuItem, "activate", G_CALLBACK(MainWindow::mnuNowyActivated), this);
	menuItem = gui->getWidget("mnuOtworz");
	g_signal_connect(menuItem, "activate", G_CALLBACK(MainWindow::mnuOtworzActivated), this);
	menuItem = gui->getWidget("mnuZakoncz");
	g_signal_connect(menuItem, "activate", G_CALLBACK(MainWindow::mnuZakonczActivated), this);
	menuItem = gui->getWidget("mnuOProgramie");
	g_signal_connect(menuItem, "activate", G_CALLBACK(MainWindow::mnuOProgramieActivated), this);
	menuItem = gui->getWidget("mnuZapiszJako");
	g_signal_connect(menuItem, "activate", G_CALLBACK(MainWindow::mnuZapiszJakoActivated), this);
	menuItem = gui->getWidget("mnuZapisz");
	g_signal_connect(menuItem, "activate", G_CALLBACK(MainWindow::mnuZapiszActivated), this);
	menuItem = gui->getWidget("mnuSkopiuj");
	g_signal_connect(menuItem, "activate", G_CALLBACK(MainWindow::mnuSkopiujActivated), this);
	
	webkit_web_view_load_uri(this->webView, this->homePage);
    gtk_widget_grab_focus(GTK_WIDGET(this->webView));
}

MainWindow::~MainWindow() {
	if (this->fileName) {
		g_string_free(this->fileName, TRUE);
	}
}

void MainWindow::saveWebPage() {
	GError *error = NULL;
	GtkWidget *statusbar = gui->getWidget("statusbar1");
	
	WebKitWebFrame *webFrame = webkit_web_view_get_focused_frame(this->webView);
	if (webFrame) {
		WebKitWebDataSource *webDataSource = webkit_web_frame_get_data_source(webFrame);
		GString *content = webkit_web_data_source_get_data(webDataSource);
		if (content) {
			if (g_file_set_contents(this->fileName->str, content->str, -1, &error)) {
				gchar *info = g_strdup_printf("Plik %s pomyślnie zapisany", this->fileName->str);
				gtk_statusbar_pop(GTK_STATUSBAR(statusbar), 0);
				gtk_statusbar_push(GTK_STATUSBAR(statusbar), 0, info);
				g_free(info);
				return;
			}
		}
	}
	
	gchar *info = g_strdup_printf("Błąd podczas zapisywania pliku %s", this->fileName->str);
	gtk_statusbar_pop(GTK_STATUSBAR(statusbar), 0);
	gtk_statusbar_push(GTK_STATUSBAR(statusbar), 0, info);
	g_free(info);
	if (error) {
		g_error_free(error);
	}
}

// ==================================================================================================
// Static functions - event handlers
// ==================================================================================================
void MainWindow::onDestroy(GtkWidget* widget, gpointer data) {
	gtk_main_quit();
}

gboolean MainWindow::onDeleteEvent(GtkWidget* widget, GdkEvent* event, gpointer data) {
	if (gui->msgBoxYesNo("Czy na pewno chcesz opuścić program?") == GTK_RESPONSE_YES) {
		return FALSE;
	}
	return TRUE;
}

gboolean MainWindow::closeWebView(WebKitWebView* webView, GtkWidget* window) {
	gtk_widget_destroy(window);
}

void MainWindow::webViewLoadStatus(WebKitWebView *webView, GParamSpec *pspec, MainWindow* window) {
	WebKitLoadStatus status = webkit_web_view_get_load_status(webView);
	GObject *object = G_OBJECT(webView);
	
	g_object_freeze_notify(object);
	
	switch (status) {
		case WEBKIT_LOAD_COMMITTED:
			gtk_entry_set_text(GTK_ENTRY(window->edtUrl), webkit_web_view_get_uri(webView));
			break;
	}
	g_object_thaw_notify(object);
}

void MainWindow::btnBackClicked(GtkWidget* button, MainWindow* window) {
    if (webkit_web_view_can_go_back(window->webView)) {
        webkit_web_view_go_back(window->webView);
    }
}

void MainWindow::btnForwardClicked(GtkWidget* button, MainWindow* window) {
    if (webkit_web_view_can_go_forward(window->webView)) {
        webkit_web_view_go_forward(window->webView);
    }
}

void MainWindow::btnGoClicked(GtkWidget* button, MainWindow* window) {
	const gchar* uri = gtk_entry_get_text(GTK_ENTRY(window->edtUrl));
	webkit_web_view_load_uri(window->webView, uri);
    gtk_widget_grab_focus(GTK_WIDGET(window->webView));
}

void MainWindow::tbOpenClicked(GtkWidget* button, MainWindow* window) {
	GtkWidget *dialog;
	dialog = gtk_file_chooser_dialog_new("Otwórz plik", GTK_WINDOW(window->wnd),
		GTK_FILE_CHOOSER_ACTION_OPEN,
		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
		GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
		NULL);
										  
	GtkFileFilter *fileFilter = gtk_file_filter_new();
	gtk_file_filter_add_pattern(fileFilter, "*.html");
	gtk_file_filter_add_pattern(fileFilter, "*.htm");
	gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(dialog), fileFilter);
	
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		gchar *filename;
		filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
		gchar *fileUri = g_strdup_printf("file://%s", filename);
		webkit_web_view_load_uri(window->webView, fileUri);
		g_free(filename);
		g_free(fileUri);
	}

	gtk_widget_destroy (dialog);
}

void MainWindow::tbHomeClicked(GtkWidget* button, MainWindow* window) {
	webkit_web_view_load_uri(window->webView, window->homePage);
    gtk_widget_grab_focus(GTK_WIDGET(window->webView));
}

void MainWindow::tbCloseClicked(GtkWidget* button, MainWindow* window) {
	gboolean retval;
	g_signal_emit_by_name(window->wnd, "delete-event", NULL, &retval);
	if (!retval) {
		gtk_widget_destroy(window->wnd);
	}
}

void MainWindow::tbAboutClicked(GtkWidget* button, MainWindow* window) {
	if (!gui->loadInterface("about_dialog.ui")) {
		return;
	}
	GtkWidget *dialog = gui->getWidget("dlgAbout");
	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);
}

void MainWindow::mnuNowyActivated(GtkWidget* menuItem, MainWindow* window) {
	GError* error = NULL;
	
	if (!g_spawn_command_line_async("./internet_viewer", &error)) {
		g_print("Błąd otwarcia programu: %s", error->message);
		g_error_free(error);
	}
}

void MainWindow::mnuOtworzActivated(GtkWidget* menuItem, MainWindow* window) {
	MainWindow::tbOpenClicked(NULL, window);
}

void MainWindow::mnuZakonczActivated(GtkWidget* menuItem, MainWindow* window) {
	MainWindow::tbCloseClicked(NULL, window);
}

void MainWindow::mnuOProgramieActivated(GtkWidget* menuItem, MainWindow* window) {
	MainWindow::tbAboutClicked(NULL, window);
}

void MainWindow::mnuZapiszJakoActivated(GtkWidget* menuItem, MainWindow* window) {
	GtkWidget *dialog;
	dialog = gtk_file_chooser_dialog_new("Zapisz jako", GTK_WINDOW(window->wnd),
		GTK_FILE_CHOOSER_ACTION_SAVE,
		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
		GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
		NULL);
										  
	GtkFileFilter *fileFilter = gtk_file_filter_new();
	gtk_file_filter_add_pattern(fileFilter, "*.html");
	gtk_file_filter_add_pattern(fileFilter, "*.htm");
	gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(dialog), fileFilter);
	
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		gchar *filename;
		filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
		if (window->fileName) {
			g_string_assign(window->fileName, filename);
		} else {
			window->fileName = g_string_new(filename);
		}
		g_free(filename);
		
		window->saveWebPage();
	}

	gtk_widget_destroy (dialog);
}

void MainWindow::mnuZapiszActivated(GtkWidget* menuItem, MainWindow* window) {
	if (window->fileName) {
		window->saveWebPage();
	} else {
		MainWindow::mnuZapiszJakoActivated(menuItem, window);
	}
}

void MainWindow::mnuSkopiujActivated(GtkWidget* menuItem, MainWindow* window) {
	webkit_web_view_copy_clipboard(window->webView);
}

void MainWindow::edtUrlActivated(GtkEntry* menuItem, MainWindow* window) {
	btnGoClicked(NULL, window);
}
